

module("NetTool_CLS", package.seeall)

local This = _M

This.NetAddrTab={}

function This:YieldParseNetAddr( NetAddr )
    local NetAddrTab=self.NetAddrTab
    local NetAddrInTab=NetAddrTab[NetAddr]
    local NetIP =nil

    if nil~=NetAddrInTab then
        NetAddrInTab.Timer:UpdateTimer()
        if NetAddrInTab.Timer:GetTimerMsec() < 5000 then
            NetIP=NetAddrInTab.NetIP
        end
    end

    if nil==NetIP then
        local ParseDomain=mx.mxParseDomain_CLS()
        ParseDomain:Init()
        local IsEnd,DomainIP

        ParseDomain:ParseStart(NetAddr)
        while true do
            ParseDomain:OnTime()
            IsEnd,DomainIP=ParseDomain:GetOutput()
            if IsEnd==1 then
                break
            end
            coroutine.yield()
        end
        if not DomainIP then--再进行解析一次
            ParseDomain:ParseStart(NetAddr)
            while true do
                ParseDomain:OnTime()
                IsEnd,DomainIP=ParseDomain:GetOutput()
                if IsEnd==1 then
                    break
                end
                coroutine.yield()
            end
        end
        if DomainIP then
            NetIP=DomainIP
            local TimerMsec=Comm.NewTimer()
            TimerMsec:ResetTimer()
            NetAddrTab[NetAddr]={NetIP=DomainIP,Timer=TimerMsec}
        end
        --如果实时解析不成功，老的有的话，用老的
        if nil== NetIP and NetAddrInTab then
            NetIP=NetAddrInTab.NetIP
        end
    end

    return NetIP
end

function This:YieldHttpRawSend( Url,Data )
    local TimerMsec =Comm.NewTimer()
    local Server=Url:match("//(.-)/")
    Server=string.split(Server,":")
    local szDomain=Server[1]
    local Port=tonumber(Server[2] or 80)
    local DomainIP=self:YieldParseNetAddr(szDomain)
    if nil==DomainIP then
        PrintString("YieldParseNetAddr Fail: "..szDomain)
        return nil
    end

    local NetClient=mx.mxNetClient_CLS()
    NetClient:Init(Comm.GetTimeMsec())
    NetClient:SetInput(DomainIP,Port)
    NetClient:Open()

    local IsConnectOK = 0
    TimerMsec:ResetTimer()
    while true do
        NetClient:OnTime()
        _,IsConnectOK=NetClient:GetOutput()
        if IsConnectOK>0 then
            break
        end
        TimerMsec:UpdateTimer()
        if TimerMsec:GetTimerMsec() >10000 then
            break
        end
        coroutine.yield()
    end

    if IsConnectOK==1 then
        NetClient:PutData(Data)
    else
         PrintString("YieldHttpRawSend Connect Fail:"..Url)
    end

    local HttpHeader=nil
    local HttpData=nil
    local RxRaw=""
    local DataLen=nil
    TimerMsec:ResetTimer()
    while IsConnectOK==1 do
        local Rx=NetClient:GetData()
        if string.len(Rx)~=0 then
            RxRaw=RxRaw..Rx
            if not DataLen then
                HttpHeader=RxRaw:match("(.-)\r\n\r\n")
                if HttpHeader then
                    Rx=HttpHeader:match("Content[-]Length: (%d+)")
                    DataLen=Rx and tonumber(Rx) or 0
                end
            end
            if DataLen and (#HttpHeader+DataLen+4)<=#RxRaw then
                HttpData=string.sub(RxRaw,#HttpHeader+5,-1)
                break
            end
        end
        TimerMsec:UpdateTimer()
        if TimerMsec:GetTimerMsec() >30000 then
            HttpHeader=nil
            HttpData=nil
            break
        end 
        NetClient:OnTime()
        _,IsConnectOK=NetClient:GetOutput()
        coroutine.yield()
    end
    NetClient:Close()

    return HttpHeader,HttpData
end

function This:CreateHttpPostPkt(Url,PostData,HeaderAdd)
    local o=nil
    local Host = string.match(Url or "",'//(.-)/')
    if Host and type(PostData)=="string" then
        o="POST "..Url.." HTTP/1.1\r\n"
        o=o.."Host: "..Host.."\r\n"
        o=o..(HeaderAdd or "")
        o=o.."Content-Length: "..PostData:len().."\r\n\r\n"
        o=o..PostData
    end
    return o
end

function This:CreateHttpGetPkt(Url,HeaderAdd)
    local o=nil
    local Host = string.match(Url or "",'//(.-)/')
    if Host then
        o="GET "..Url.." HTTP/1.1\r\n"
        o=o..(HeaderAdd or "")
        o=o.."Host: "..Host.."\r\n\r\n"
    end
    return o
end

function This:WaitTcpSend(NetIpAddr,Port,Data,TxTimeOut,RxTimeOut)    
    mx=require("mx")
    local Result=nil
    local TxTimeOut = tonumber(TxTimeOut)
    local RxTimeOut = tonumber(RxTimeOut) or 0

    local m_CurTimeMsec=mx.mxTimeMsec_CLS()
    local TimerMsec = mx.mxTimerMsec_CLS()
    local DomainIP=mx.mxGetIP(NetIpAddr)
    local NetClient=mx.mxNetClient_CLS()

    TimerMsec:Init(m_CurTimeMsec)
    TimerMsec:ResetTimer()
    NetClient:Init(m_CurTimeMsec)
    NetClient:SetInput(DomainIP,tonumber(Port))
    NetClient:Open()

    local IsConnectOK = 0
    while true do
        NetClient:OnTime()
        _,IsConnectOK=NetClient:GetOutput()
        if IsConnectOK>0 then
            break
        end
        TimerMsec:UpdateTimer()
        if TimerMsec:GetTimerMsec() >5000 then
            break
        end
        mx.mxSleep(50)
        m_CurTimeMsec:AddVal(50)
    end

    local TotalRx={}
    local Rx
    --发送
    TimerMsec:ResetTimer()
    while IsConnectOK==1 do
        local n=NetClient:PutData(Data)
        if n~=#Data then
            Data=string.sub(n+1,-1)
        else
            break
        end
        Rx=NetClient:GetData()
        if string.len(Rx)~=0 then
            table.insert(TotalRx,Rx)
            break
        end
        TimerMsec:UpdateTimer()
        if TxTimeOut and TxTimeOut~=0 and TimerMsec:GetTimerMsec()>TxTimeOut then
            break
        end 
        NetClient:OnTime()
        _,IsConnectOK=NetClient:GetOutput()
        mx.mxSleep(50)
        m_CurTimeMsec:AddVal(50)
    end
    --接收
    TimerMsec:ResetTimer()
    while IsConnectOK==1 do
        Rx=NetClient:GetData()
        if string.len(Rx)~=0 then
            table.insert(TotalRx,Rx)
            break
        end
        TimerMsec:UpdateTimer()
        if TimerMsec:GetTimerMsec()>RxTimeOut then
            break
        end
        NetClient:OnTime()
        _,IsConnectOK=NetClient:GetOutput()
        mx.mxSleep(50)
        m_CurTimeMsec:AddVal(50)
    end
    Result=table.concat(TotalRx)
    NetClient:Close()

    return Result
end

return _M

--[[
function This:sendAlarmTemplateMsg(Url,MsgObj)
    if Url and MsgObj then
        local PostData=json.encode(MsgObj)
        local HttpRawData=self:CreateHttpPkt(Url,PostData)
        self.Ref.addTask(coroutine.create(function ()
            PrintString(HttpRawData)
            local Res=self:httpRawSendUseYield(Url,HttpRawData)
            if Res then
                PrintString(Res)
            end
        end))
    end
end

function This:sendWeiXinText(WeiXinServerID,WeiXinOpenID,Text)
    while true do
        local  AccessToken=self.Ref.getAccessToken(WeiXinServerID)
        if nil==AccessToken then
            PrintString("No AccessToken for WeiXinServerID="..WeiXinServerID)
            break
        end
        local  touser=WeiXinOpenID
        local PostData = string.format(
            '{"Type":1,"AccessToken":"%s","Data":{"touser":"%s","msgtype":"text","text":{"content":%s}}}',
            AccessToken,touser,json.encode(Text) )
        local Url = self.Ref.getWeiXinSeverURL(WeiXinServerID)
        local HttpRawData = self:CreateHttpPkt(Url,PostData)
        self.Ref.addTask(coroutine.create(function ()
            PrintString(HttpRawData)
            local Res=self:httpRawSendUseYield(Url,HttpRawData)
            if Res then
                PrintString(Res)
            end
        end))
        break
    end
end
--]]