--[[
	1.NetAddr,Port,HeardbeatInterval是程序里固定写的

	1.调用Connect连接服务器。
	2.调用Login登录服务器，模块内部会处理心跳。
	3.SendPktToserver存为队列，在合适时发到服务器。
	4.服务器返回通过回调给出。
	5.通信断开或登录失败通过回调给出，并终止未完成队列中的所有内容。
	6.如果登录过程中需要验证码，通过回调提示并待输入验证码后，继续未完成队列中数据发送。
	7.如果中途重新调用Connect或Login,则之前的未完成的任务立即终止并全新开始。
	8.退出来原始状态，有三种可能：a.连接不成功;b.登录不成功;c.中途连接断开;d.退出登录;d.关闭连接;


	1.发给服务器的报文分为两种：1.未登录前就可以发给服务器的；2.登录后才发给服务器的。	
	2.这两种报文在接口函数里初步处理，不正常的以立即返回方式报告，符合条件的存为队列并在协程处理函数时依次处理。
	3.初步处理时，如果没有连服务器，启动ConnectStart。
	4.在ConnectStart里创建协程，协程里在有需要时自动连接服务器或自动登录服务器。


	1.在StartLogin时，生成一个处理协程，并在此协程里连接服务器，并尝试登录，如果登录不成功协程退出。
	2.协程退出时，清除协程号成员变量，并报相应的出错提示。
	3.当协程号不存在时，表示不会自动重连，需要外部调用StartLogin才可以重新登录服务器。
	4.当协程号存在时，但状态为未登录成功，表示即将登录或正正登录中。
	5.成员里有一个协程专用的表，存放当前工作协程的参数，同时程里有一个此表的upval。
	6.协程处理数据时，检测upval里的协程号是否与self里的协程号相等，不相等时退出当前协程。
	7.本模块的状态码：0.未登录状态；1.已TCP连接状态;2.正在登录状态;3.已登录状态。
	8.发给服务器的报文，会进行队列中，等到适合的时机发送，队列的有效期为登录出错之前。
	9.发给服务器的报文有两为在，1.未登录前就可以发给服务器的；2.登录后才发给服务器的。	
]]

local NetTool_CLS=require("NetTool")

local This
if App then
	This=App.m_AccountManager
else
	This={} --方便单文件检测语法错误
end

function This:Init()
	self.m_DevEventInfoCachPktList=List.newList()
end

function This:OnTime()
	--正常退出时,self.m_CoParam.m_Coroutine会被设为nil，不正常退出时，给出提示
	if self.m_CoParam and self.m_CoParam.m_Coroutine and coroutine.status(self.m_CoParam.m_Coroutine)=="dead" then
		self:OnCoroutineNotify(self.m_CoParam,"CoroutineDead")
		if self.m_CoParam.m_NetClient then
			self.m_CoParam.m_NetClient:Close()
		end 
		self.m_CoParam=nil
	end
	--触发回应
	local CoParam=self.m_CoParam
	self:ServerProxyOnRx(CoParam,nil)
end

function This:Connect(NetAddr,Port)
	local CoParam={
		m_Coroutine=nil,
		m_IswaitRtHeartbeat=false,		
		m_TaskList=List.newList(),
		m_TimerForSendData=nil,
		m_NetClient=nil,
	}
	self.m_DevEventInfoCachPktList.clear()
	--创建处理协程
	CoParam.m_Coroutine=coroutine.create(function()
		local TimerMsec=mx.mxTimerMsec_CLS()
		local NetClient=mx.mxNetClient_CLS()
		while true do--方便break跳出,处理协程
			--local DomainIP=mx.mxGetIP(NetAddr)
			local DomainIP=NetTool_CLS:YieldParseNetAddr(NetAddr)
			CoParam.m_NetClient=NetClient
			TimerMsec:Init(Comm.GetTimeMsec())
			NetClient:Init(Comm.GetTimeMsec())
			NetClient:SetInput(DomainIP or 0,Port)
			---------------连接服务器-----------------
			local IsConnectOK = 0
			for i=1,2 do--最多偿试两次连接服务器
				TimerMsec:ResetTimer()
				NetClient:Open()
				while true do
					NetClient:OnTime()
					_,IsConnectOK=NetClient:GetOutput()
					if IsConnectOK>0 then
						break
					end
					TimerMsec:UpdateTimer()
					if TimerMsec:GetTimerMsec()>5000 then
						break
					end
					coroutine.yield()
				end
				if IsConnectOK>0 then
					break
				end
			end
			if IsConnectOK>0 then
				self:OnCoroutineNotify(CoParam,"ConnectOK")
			else
				self:OnCoroutineNotify(CoParam,"ConnectFail")
				break
			end
			-----------生成心跳报文-----------
			local HeardbeatInterval=30000
			local HeardbeatPkt = {PktType="Heartbeat"}
			HeardbeatPkt=json.encode(HeardbeatPkt)
			HeardbeatPkt =LxATP.JsonToPkt(HeardbeatPkt,'$')
			---------------协程循环-------------
			CoParam.m_IswaitRtHeartbeat=false
			local TimerForRtHeartbeat=mx.mxTimerMsec_CLS()
			TimerForRtHeartbeat:Init(Comm.GetTimeMsec())
			local TimerForRtLogin=mx.mxTimerMsec_CLS()
			TimerForRtLogin:Init(Comm.GetTimeMsec())
			while true do
				NetClient:OnTime()
				--心跳的发和收
				TimerMsec:UpdateTimer()
				if TimerMsec:GetTimerMsec()>HeardbeatInterval then
					TimerMsec:ResetTimer()
					self:ServerProxyYieldSendPkt(CoParam,HeardbeatPkt)
					TimerForRtHeartbeat:ResetTimer()
					CoParam.m_IswaitRtHeartbeat=true
				end
				if CoParam.m_IswaitRtHeartbeat then
					TimerForRtHeartbeat:UpdateTimer()
					if TimerForRtHeartbeat:GetTimerMsec()>20*1000 then
						PrintString("Wait RtHeartbeat Out Of Time")
						break
					end
				end
				--处理队列任务
				while true do
					--接收数据并处理
					while true do
						local Rx=NetClient:GetData()
						if string.len(Rx)~=0 then
							self:OnRxData(CoParam,Rx)
							NetClient:OnTime()
							TimerForRtHeartbeat:ResetTimer()
						else
							break
						end
					end
					--发送队列中数据
					local Task=CoParam.m_TaskList:popFront()
					if Task then
						self:ServerProxyYieldSendPkt(CoParam,Task)
					else
						break
					end
					--检测是否已断开
					_,IsConnectOK=NetClient:GetOutput()
					if 0==IsConnectOK then
						break
					end
				end
				_,IsConnectOK=NetClient:GetOutput()
				if 0==IsConnectOK then
					PrintString("TCP Break Down")
					break
				end
				--检测是否需要主动退出
				if self.m_CoParam~=CoParam then
					break
				end
				coroutine.yield()
			end--协程循环
			self:OnCoroutineNotify(CoParam,"LinkBreak")
			break--处理协程
		end
		NetClient:Close()
		if self.m_CoParam==CoParam then
			self.m_CoParam=nil
		end
	end)
	Comm.AddTaskCoroutine(CoParam.m_Coroutine)
	self.m_CoParam=CoParam
end

function This:DisConnect()
	self.m_CoParam=nil
end

function This:SendPktToServer(Pkt)
	if self.m_CoParam and self.m_CoParam.m_TaskList then
		self.m_CoParam.m_TaskList.pushBack(Pkt)
	else
		PrintString("SendPktToServer() must after Connect")
		local Tx=[[{"CmdType":"Notify","StatusCode":3,"Status":"SendPktToServer() must after Connect"}]]
		self.Ref.CallBackOnReceive(1,Tx)
	end
end

function This:YieldSendPkt(CoParam,Pkt)
	local Data=Pkt
	--发送
	local TimerMsec = CoParam.m_TimerForSendData
	if not TimerMsec then
		TimerMsec = mx.mxTimerMsec_CLS()
		CoParam.m_TimerForSendData=TimerMsec
		TimerMsec:Init(Comm.GetTimeMsec())
	end
	local NetClient = CoParam.m_NetClient
	local Result = false
	if TimerMsec and NetClient then
		TimerMsec:ResetTimer()
		local _,IsConnectOK=NetClient:GetOutput()
		while IsConnectOK==1 do
			local n=NetClient:PutData(Data)
			if n~=#Data then
				Data=string.sub(Data,n+1,-1)
			else
				Result=true
				break
			end
			TimerMsec:UpdateTimer()--一包报文超过10秒还发不完，认为已断开。
			if TimerMsec:GetTimerMsec()>10000 then
				NetClient:Close()
				break
			end	
			NetClient:OnTime()
			_,IsConnectOK=NetClient:GetOutput()
			coroutine.yield()
		end
	end
	return Result
end

function This:OnRxData(CoParam,Data)
	local RxData=(CoParam.m_RxData or "")..Data
	while true do
		if string.len(RxData)<10 then
			break
		end
		local LenH,LenL=string.byte(RxData,3,4)
		local Len=LenH*256+LenL	
		local Header=string.sub(RxData,1,2)
		if Len<10 or (Header~="$$" and Header~="@@") then
			RxData=string.sub(RxData,2,-1)--数据去掉一字节
			PrintString("Get Error Pkt Header")
		else
			if string.len(RxData)<Len then--等到收齐报文数据
				break
			end
			local Pkt = string.sub(RxData,1,Len)
			RxData=string.sub(RxData,Len+1,-1)
			--
			if string.sub(Pkt,5,6)==string.char(0xA0,0x80) then
				local MsgJson=LxATP.PktToJson(Pkt)
				local MsgObj=json.decode(MsgJson)
				self:ServerProxyOnRx(CoParam,MsgObj)
			end		
		end
	end
	CoParam.m_RxData=RxData
end

function This:OnRxPkt(CoParam,Pkt)
	if string.sub(Pkt,5,6)==string.char(0xA0,0x80) then
		local RawPkt=LxATP.PktToJson(Pkt)
		if RawPkt and self.m_CoParam==CoParam then
			PrintString("RxServer:"..RawPkt)
			local PktObj=json.decode(RawPkt)
			local IsProcessed=false
			for k,v in pairs(self.m_RxFuncTable) do
				local is=k(PktObj)
				if is then
					IsProcessed=true
				end
			end
			CoParam.m_IswaitRtHeartbeat=false
			if PktObj.PktType~="RtHeartbeat" and IsProcessed==false then
				self.Ref.CallBackOnReceive(2,RawPkt)
			end
		end
	end
end

function This:OnCoroutineNotify(CoParam,Status)
	if self.m_CoParam==CoParam then
		local Tx = nil
		if "ConnectFail"==Status then
			Tx=[[{"CmdType":"Notify","StatusCode":1,"Status":"Connect Fail"}]]
		elseif "ConnectOK"==Status then
			Tx=[[{"CmdType":"Notify","StatusCode":2,"Status":"Connect OK"}]]
		elseif "LinkBreak"==Status then
			Tx=[[{"CmdType":"Notify","StatusCode":3,"Status":"Link Break"}]]
		elseif "CoroutineDead"==Status then
			Tx=[[{"CmdType":"Notify","StatusCode":3,"Status":"Break Link"}]]
			PrintString("m_AccountManager.m_Coroutine dead")
		else
			PrintString("Unkwon Status:"..Status)
		end
		if Tx then
			self.Ref.CallBackOnReceive(1,Tx)
		end
	end
end

function This:SetRxFunc(Func,Valid) 
	self.m_RxFuncTable[Func]=Valid
end

function This:ServerProxyYieldSendPkt(CoParam,Pkt)
	local MsgJson=LxATP.PktToJson(Pkt)
	local Data=Pkt
	local MsgObj=json.decode(MsgJson)
	--识别出如果是发给设备的，报文里的设备报文没有算校验，自动加上。	
	if type(MsgObj.PktType)=="string" and "MsgToDev"==MsgObj.PktType and type(MsgObj.MsgData)=="string" then
		local MsgDev=string.hex2bin(MsgObj.MsgData)
		MsgDev=string.sub(MsgDev,5,-5)
		local PktDev=LxATP.RawDataToPkt(MsgDev,'@')
		MsgObj.MsgData=string.bin2hex(PktDev)
		local MsgJson=json.encode(MsgObj)
		Data=LxATP.JsonToPkt(MsgJson,'$')
	end
	--根据本地缓存是否存在决定是否发给服务器。
	if "GetDevEventInfo"==MsgObj.PktType then
		local EventID=MsgObj.EventID
		local EventInfo=self.Ref.GetDevEventInfo({EventID=EventID})
		if EventInfo then
			PrintString("Cached EventID="..EventID)
			EventInfo.DevID=MsgObj.DevID
			RtPktObj={PktType="RtGetDevEventInfo",StatusCode=0,Status="OK",EventInfo=EventInfo,No=MsgObj.No}
			self.m_DevEventInfoCachPktList.pushBack({PktType="Rt"..MsgObj.PktType,Time=os.time(),PktObj=RtPktObj})
		else
			PrintString("NotCached EventID="..EventID)
			self.m_DevEventInfoCachPktList.pushBack({PktType="Rt"..MsgObj.PktType,Time=os.time(),PktObj=nil})			
			--PrintString("TxServer:"..MsgJson)
			This:YieldSendPkt(CoParam,Data)
		end
	else
		PrintString("TxServer:"..MsgJson)
		This:YieldSendPkt(CoParam,Data)
	end
end

function This:ServerProxyOnRx(CoParam,PktObj)
	if PktObj then
		--确认期望收到的报文
		if "RtGetDevEventInfo"==PktObj.PktType then
			if type(PktObj.EventInfo)=="table" then
				self.Ref.SaveDevEventInfo(PktObj.EventInfo)
			end
			self.m_DevEventInfoCachPktList.popFront()
		end
		--向上层发
		local MsgJson=json.encode(PktObj)
		local Pkt=LxATP.JsonToPkt(MsgJson,'@')		
		self:OnRxPkt(CoParam,Pkt)
	end
	while true do
		local ExpectRxPkt=self.m_DevEventInfoCachPktList.popFront()
		if not ExpectRxPkt then
			break
		end
		if ExpectRxPkt.PktObj then--有缓存时向上层送出
			local MsgJson=json.encode(ExpectRxPkt.PktObj)
			local Pkt=LxATP.JsonToPkt(MsgJson,'@')
			PrintString("Next One Is From Cache:")
			PrintString("Callback EventID"..ExpectRxPkt.PktObj.EventInfo.EventID)
			self:OnRxPkt(CoParam,Pkt)
		else--没有缓存时，超时去期望报文
			local Time=os.time()
			if (ExpectRxPkt.Time+60)>Time then
				self.m_DevEventInfoCachPktList.pushFront(ExpectRxPkt)
				break
			else
				--在预期报文超过1分钟则无效，仍掉
				PrintString("ExpectRxPkt Out Of Time")
			end
		end
	end
end
