
require("appComm")
require "luasql.sqlite3"

local This
if App then
	This=App.m_Sqlite
else
	This={}
end
This.Static=This

local SqliteDbFile=LuaPath and LuaPath.."/sqlite3.db" or "sqlite3.db"

function This:Init()
    local MaxEntrys=20000--指定t_dev_event_info最多缓存多少条
    --如果没有表，建立对应的表
    local env,conn=self:GetSqlEnvConn()
    local Sql,Cur
    Sql=string.format([[SELECT event_id,event_type,time,time_str,msg_data FROM t_dev_event_info WHERE id="";]])
    Cur=conn:execute(Sql)
    if nil==Cur then
        Sql=[[DROP TABLE IF EXISTS "main"."t_dev_event_info";]]
        Cur=conn:execute(Sql)
        Sql=[[
CREATE TABLE "t_dev_event_info" (
"id"  INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
"event_id"  TEXT NOT NULL,
"event_type"  TEXT,
"time"  INTEGER,
"time_str"  TEXT,
"msg_data"  TEXT,
UNIQUE ("event_id")
);]]
        Cur=conn:execute(Sql)
    end
    --保留最近加入的MaxEntrys条记录，将旧的删除掉
    while true do
        Sql=string.format([[SELECT id FROM t_dev_event_info ORDER BY id DESC LIMIT %s,1;]],MaxEntrys)
        local t=ExecuteOneSql(conn,Sql)
        if not t[1] then
            break
        end
        --删除多出的部分
        Sql=[[DELETE FROM t_dev_event_info WHERE id <= %s;]]
        Sql=Sql:format(t[1][1])
        Cur= conn:execute(Sql)
    break end
end

function This:OnTime()
    if self.m_IsBegin then
        local conn=self.m_SqlConn
        conn:execute("COMMIT;")
        self.m_IsBegin=false
    end
end

function This:GetSqlEnvConn()
	local conn=self.m_SqlConn
	local env =self.m_SqlEnv
	local Cur = nil
	--如果当前的不正常，重新连
	if not conn then
		if conn and conn:close() then end
		if env and env:close() then end
		local luasql=require "luasql.sqlite3"
		env = luasql.sqlite3()
		conn = env:connect(SqliteDbFile)
		self.m_SqlConn=conn
		self.m_SqlEnv=env
	end
    --标识是否已启动
    if (not self.m_IsBegin) and conn then
        Cur=conn:execute("BEGIN;")
        if Cur then
            self.m_IsBegin=true
        end
    end
	return env,conn
end

function ExecuteOneSql(conn,Sql)
    local Result={}
    local Row,Cur   
    Cur = conn:execute(Sql)
    assert(Cur,Sql)
    Row = Cur:fetch({})
    while Row do
        table.insert(Result,Row)
        Row={}
        Row=Cur:fetch(Row)
    end
    return Result
end

function This:SaveDevEventInfo(Obj)
    --Obj如：{"EventID":"AB879","EventType":"JSON","Time":"2015-11-02 16:02:09","MsgData":"此区域为JSON格式的服务器报文"}
    local env,conn=self:GetSqlEnvConn()
    local Rt=nil
    while true do
        local EventID=conn:escape(Obj.EventID)
        local EventType=conn:escape(Obj.EventType)
        local Time=conn:escape(Obj.Time)
        local MsgData=conn:escape(Obj.MsgData)
        --插入数据库
        local Cur,Sql
        Sql=[[INSERT INTO t_dev_event_info (event_id,event_type,time,time_str,msg_data) VALUES ("%s","%s",strftime('%%s','%s'),'%s','%s');]]
        Sql=Sql:format(EventID,EventType,Time,Time,MsgData)
        Cur = conn:execute(Sql)
        if nil==Cur then break end
        Rt=0
    break end    
    return Rt
end

function This:GetDevEventInfo(Obj)
	local env,conn=self:GetSqlEnvConn()
    Rt=nil
    local EventID = conn:escape(Obj.EventID)
    local TimeZoneSec=Comm.TimeZoneSec
    while true do
        local Cur,Sql
        Sql=string.format([[SELECT event_id,event_type,time_str,msg_data FROM t_dev_event_info WHERE event_id="%s";]],EventID)        
        local t=ExecuteOneSql(conn,Sql)
        if not t[1] then
            break
        end
        t=t[1]
        Rt={
            EventID=t[1],
            EventType=t[2],
            Time=t[3],
            MsgData=t[4],
        }
   break end
   return Rt
end

function This:SqliteTest(Obj)
    local StrTime=""
    local env,conn=self:GetSqlEnvConn()
    local Cur
    while true do
        local Cur,Sql
        Sql=string.format([[SELECT event_id,id FROM t_dev_event_info;]])
        local TableEventID=ExecuteOneSql(conn,Sql)
        StrTime=string.format("Begin:%s,%s,%s",os.time(),os.clock(),#TableEventID)
        if PrintString then
            PrintString(StrTime)
        end
        print(StrTime)
        for i,v in ipairs(TableEventID) do
            local EventID=v[1]
            self:GetDevEventInfo({EventID=EventID,ID=v[2]})
        end
        StrTime=string.format("End:%s,%s",os.time(),os.clock())
        if PrintString then
            PrintString(StrTime)
        end
        print(StrTime)
   break end
end


--[=[
local Str=[[{"EventID":"AB87","EventType":"JSON","Time":"2015-11-02 16:02:09","MsgData":"此区域为JSON格式的服务器报文"}]]
local Obj =json.decode(Str)
local Rt=This:SaveDevEventInfo(Obj)
print(Rt)

Rt=This:GetDevEventInfo({EventID="AB87"})
print(json.encode(Rt))

print(Comm.TimeZoneSec)
]=]
--This:Init()
--This:SqliteTest()